//
//  ReaderViewController.m
//  library
//
//  Created by user on 2012-03-21.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "ReaderViewController.h"
#import "LibAppDelegate.h"
#import "BookViewController.h"
#import "Reader.h"


@interface ReaderViewController()

@property (nonatomic, strong) NSFetchedResultsController *fetchedResultsController;
@end

@implementation ReaderViewController

@synthesize contextPtr = _contextPtr;
@synthesize fetchedResultsController = _fetchedResultsController;

- (void)viewDidLoad {
    self.contextPtr = [(LibAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Reader"];
    
    NSSortDescriptor *descriptor1 = [NSSortDescriptor sortDescriptorWithKey:@"surname" ascending:YES comparator:^(NSString *obj1, NSString *obj2) {
        return [obj1 compare:obj2 options:NSCaseInsensitiveSearch];}];
    NSSortDescriptor *descriptor2 = [NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES comparator:^(NSString *obj1, NSString *obj2) {
        return [obj1 compare:obj2 options:NSCaseInsensitiveSearch];}];
    
    request.sortDescriptors = [NSArray arrayWithObjects:descriptor1, descriptor2, nil];
    
    self.fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:request
                                                                        managedObjectContext:self.contextPtr
                                                                          sectionNameKeyPath:nil
                                                                                   cacheName:nil];
    self.fetchedResultsController.delegate = self;
    [self.fetchedResultsController performFetch:nil];
}

- (void)dismissAddReaderViewController {
    [self dismissModalViewControllerAnimated:YES];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([segue.identifier isEqualToString:@"AddReaderSegue"]) {
        AddReaderViewController *destination = segue.destinationViewController;
        
        destination.delegate = self;
        destination.contextPtr = self.contextPtr;
    } else {
        BookViewController *destination = segue.destinationViewController;
        
        destination.contextPtr = self.contextPtr;
        destination.reader = [self.fetchedResultsController objectAtIndexPath:self.tableView.indexPathForSelectedRow];
    }
}

#pragma mark - UITableViewDataSource methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return [self.fetchedResultsController.sections count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [[self.fetchedResultsController.sections objectAtIndex:section] numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Reader Cell"];
    
    NSString *readerIdentity = [[self.fetchedResultsController objectAtIndexPath:indexPath] valueForKey:@"surname"];
    readerIdentity = [readerIdentity stringByAppendingFormat:@" %@", [[self.fetchedResultsController objectAtIndexPath:indexPath] valueForKey:@"name"]];
    
    cell.textLabel.text = readerIdentity;
    
    return cell;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [self.contextPtr deleteObject:[self.fetchedResultsController objectAtIndexPath:indexPath]];
        [(LibAppDelegate *)[[UIApplication sharedApplication] delegate] saveContext];
    }
}

#pragma mark - NSFetchedResultsControllerDelegate methods

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type newIndexPath:(NSIndexPath *)newIndexPath {
    switch (type) {
        case NSFetchedResultsChangeInsert: {
            [self.tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            [self.tableView scrollToRowAtIndexPath:newIndexPath atScrollPosition:UITableViewScrollPositionTop animated:YES];
            break;
        }
        case NSFetchedResultsChangeDelete: {
            [self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
        }
            
        default:
            break;
    }
}

@end
